/*
**  ClanLib SDK
**  Copyright (c) 1997-2020 The ClanLib Team
**
**  This software is provided 'as-is', without any express or implied
**  warranty.  In no event will the authors be held liable for any damages
**  arising from the use of this software.
**
**  Permission is granted to anyone to use this software for any purpose,
**  including commercial applications, and to alter it and redistribute it
**  freely, subject to the following restrictions:
**
**  1. The origin of this software must not be misrepresented; you must not
**     claim that you wrote the original software. If you use this software
**     in a product, an acknowledgment in the product documentation would be
**     appreciated but is not required.
**  2. Altered source versions must be plainly marked as such, and must not be
**     misrepresented as being the original software.
**  3. This notice may not be removed or altered from any source distribution.
**
**  Note: Some of the libraries ClanLib may link to may have additional
**  requirements or restrictions.
**
**  File Author(s):
**
**    Magnus Norddahl
**    Mark Page
**    Artem Khomenko
*/

#pragma once

class Theme
{
public:
	static std::shared_ptr<clan::ScrollBarView> create_scrollbar(bool isHorizontal = true);
	static void initialize_scrollbar(const std::shared_ptr<clan::ScrollBarView> &scrollbar, bool isHorizontal);

	static std::shared_ptr<clan::ButtonView> create_button();
	static std::shared_ptr<clan::SliderView> create_slider();
	static std::shared_ptr<clan::CheckBoxView> create_checkbox();
	static std::shared_ptr<clan::RadioButtonView> create_radiobutton();
	static std::shared_ptr<clan::LabelView> create_label(bool subpixel = false);
	static std::shared_ptr<clan::ListBoxView> create_listbox();
	static std::shared_ptr<clan::LabelView> create_listbox_label(const std::string &text);
};
